
local fmt=string.format
local quote=luasql.quotestr
local sqlutil=require"sqlutil"


-- Use for DELETE, INSERT or UPDATE
local function sqlexec(conn,sql)
   local ok,e1,e2
   local cnt=10
   while cnt > 0 do
      ok,e1,e2=conn:execute(sql)
      if not ok then
         if e1 ~= "BUSY" then
            trace("SQL maillist err:",sql," : ",e1,e2)
            break
         end
      else
         return ok,e1,e2
      end
      collectgarbage()
      ba.sleep(1000)
      cnt=cnt-1
   end
   return ok,e1,e2
end


local function settings(conn,key,val)
   if val then
      return sqlexec(conn,fmt("UPDATE mailsettings SET value=%s WHERE key=%s",quote(val),quote(key)))
   end
   return sqlutil.find(conn,"value FROM mailsettings WHERE key="..quote(key))
end
   

local function insertcontact(conn,listkey,email,firstname,lastname)
   email,firstname,lastname = string.lower(bd.trim(email)),bd.trim(firstname),bd.trim(lastname)
   local time=os.time()
   local sha1=ba.crypto.sha1()
   sha1(tostring(ba.clock()))
   sha1(email)
   sha1(tostring(time))
   sha1(tostring(ba.rnd()))
   local hash=ba.b64encode(sha1()):gsub("[+/=]","")
   local sql=
      fmt("%s VALUES(%s,%s,%s,'%s',%u,%u)",
          "INSERT INTO mailcontact (email,firstname,lastname,hash,listkey,time)",
          quote(email),
          quote(firstname),
          quote(lastname),
          hash,
          listkey,
          time)
   return sqlexec(conn,sql)
end


local function findemail(conn,email)
   return sqlutil.find(conn,fmt("DISTINCT key FROM mailcontact WHERE email=%s collate NOCASE",quote(email)))
end


local function expandmacros(body, actionT)
   if body then
      local foundT={}
      local function expand(data)
         local macro=data:sub(2,-2)
         local action=actionT[macro]
         if action then
            foundT[macro] = foundT[macro] and foundT[macro]+1 or 1
            return action(macro)
         end
         return data
      end
      return body:gsub("(%[[^%[%]]+%])", expand),foundT
   end
end


local fileMailerT

local function loadMailer(name)
   if not fileMailerT or not fileMailerT[name] then
      fileMailerT=ba.openio"vm":dofile(".bdlua/filemailer.lua")
   end
end

local function startjob(...)
   loadMailer'startjob'
   return fileMailerT.startjob(...)
end

do
   local curjobkey
   local cursendcnt


   local lasterrmsg;
   return {
      sqlexec=sqlexec,
      settings=settings,
      insertcontact=insertcontact,
      findemail=findemail,
      startjob=startjob,
      expandmacros=expandmacros,
      seterr=function(err) lasterrmsg=err end,
      geterr=function(reset) if reset then lasterrmsg=nil else return lasterrmsg end end,
      setjob=function(key,sendcnt) curjobkey,cursendcnt=key,sendcnt end,
      getjob=function() return curjobkey,cursendcnt end,
      sendmail=function(...) loadMailer'sendmail' return fileMailerT.sendmail(...) end
   }
end
