<?lsp

local fmt=string.format
local tinsert=table.insert

local curUri=request:uri()

local function toHref(name,uri,selT)
   if type(uri) == "string" then
      if uri == curUri then
         if selT then selT.selected=true end
         return fmt("<li><a href='%s' class='selected'>%s</a></li>",uri,name)
      end
      return fmt("<li><a href='%s'>%s</a></li>",uri,name)
   end
   local selT2={}
   local t={}
   local uris=uri.ut
   for k,v in ipairs(uri.nt) do
      tinsert(t,toHref(v,uris[k],selT2))
   end
   tinsert(t,"</ul></li>")
   if selT and selT2.selected then selT.selected=selT2.selected end
   return fmt("<li><span%s>%s</span><ul>",selT2.selected and
              " class='selected'" or "",
              name)..table.concat(t)
end



local function emitLi(names,uris)
   for k,v in ipairs(names) do
      write(toHref(v,uris[k]))
   end
end


local function emitArchive()
   local month=cmsMonth
   for _,year in ipairs(app.archList) do
      local y = year.y
      write(toHref(fmt("<b>%s</b>",y),"/blog/"..y))
      for _,m in ipairs(year.m) do
         write(toHref(month(m),fmt("/blog/%s/%s",y,m)))
      end
   end
end


local function emitContent()
   if cmsBlog then
      local permalink,id,date,cat,subj,name,data=cmsBlog()
      while permalink do
         local pi = cmsPostedIn(cat)
         write("<h2><a href='",
                        permalink,
                        "'>",
                        subj,
                        "</a></h2>",
                        data,
                        "<p>",
                        pi and pi or "",
                        os.date(" %A, %B %dth, %Y ", date),
                        #name > 0 and (" by "..name) or "",
                        "</p>")
         local title,fbKey=cmsFbComments()
         if title then
            if fbKey then
               write'<a name="fbcomments"></a>'
               app.emitFbCommentBox(write,id,fbKey)
            else
               write("<p><a href='",permalink,"#fbcomments'>",title,"</a></p>")
            end
         end
         permalink,id,date,cat,subj,name,data=cmsBlog()
      end
   else
      cmsEmitPage()
   end
end

?>


<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01//EN" "http://www.w3.org/TR/html4/strict.dtd">
<html xmlns="http://www.w3.org/1999/xhtml" xmlns:fb="http://www.facebook.com/2008/fbml">
<head>
<title><?lsp=cmsTitle?></title>
<meta http-equiv="Content-Type" content="text/html; charset=utf-8"/>
<link rel="icon" href="/rtl/favicon.ico" type="image/x-icon" />
<link href="/theme/bd.css" rel="stylesheet" type="text/css" />
<?lsp=cmsHeader?>
<script src="/theme/bd.js" type="text/javascript"></script>
</head>
<body>   
<div id="head">
</div>
<table id="nav">
<tr><td rowspan="2">
   <img src='<?lsp=bd.logoURL()?>' alt='logo'/>
</td>
<td>
<ul>
<li><a class='selected' href='/'>Home Page</a></li>
<?lsp
local links = {
  {"Web-File-Server", "/rtl/protected/wfslinks.lsp"},
  {"Menu", "/rtl/protected/"},
  {"CMS Admin", "/private/manage/"},
  {"About", "/rtl/about.lsp"},
}
if request:user() then
  tinsert(links, {"Logout", "/rtl/logout.lsp"})
   enablemenuhelp = not bd.cfg.load().v.disablemenuhelp and true or false
   page.enablemenuhelp=enablemenuhelp
else
   enablemenuhelp=page.enablemenuhelp
end
for _,v in pairs(links) do
    response:write("<li><a href='",v[2],"'>",v[1],"</a></li>\n")
end
?>
</ul>
</td></tr>
<tr><td></td></tr>
</table>
<div id="wrapper">
  <div id="container">
    <div id="content">
      <?lsp emitContent() ?>
      <br />
    </div><!--content-->
  </div><!--container-->
  <div id="sidebar">
<?lsp if cmsShowAdmLinks then ?>
<h2 style="color:white">Edit Page</h2><div style="margin:1em"><div id="cmsManage"></div></div>
<?lsp end ?>

    <ul>
    <li><span class="menuitem">Links</span></li>
    <?lsp
       emitLi(app.links.names, app.links.uris)
       if cmsIsBlog then
          if app.cats then
             print'<li><span class="menuitem">Categories</span></li>'
             emitLi(app.cats.names,app.cats.uris)
          end
          print'<li><span class="menuitem">Archive</span></li>'
          emitArchive()
       end
    ?>
    <li><span class="menuitem">Loaded Applications</span></li>
    <?lsp
    local links = {}
    for k,v in pairs(bd.appmgr.list) do
       if v.running == true and not v.rootapp and not v.hidden then
          local desc = v.description or ""
          tinsert(links, {k, "/"..k.."/", desc})
       end
    end

    for _,v in pairs(links) do
       if v[1] == title then
          response:write("<li><a class='selected' href='",v[2],"'>",v[1],
                         "<span>You are here!</span></a></li>")
       else
          response:write("<li><a href='",v[2],"'>",v[1])
          if enablemenuhelp then
             response:write('<span>',v[3],'</span>')
          end
          response:write('</a></li>')
       end
    end
    ?>
    </ul>
  </div>
</div>
<div id="rtlinfo">
<p>Powered by the <a target="rtl" href="http://realtimelogic.com/products/barracuda-application-server/">Barracuda Application Server</a> and the <a target="rtl" href="http://realtimelogic.com/products/sharkssl/">SharkSSL Embedded SSL Stack</a></p>
</div>
<?lsp cmsEmitGA() ?>
</body>
</html>
