
local app=(function(app) return app end)(...)

local trace=trace

-- Saved upvalues for the theme
local themeName
local themeFunc
local themePage
local themeDir

local sfind=string.find
local fmt=string.format

local function parselsp(data, name)
   local x
   x,err = ba.parselsp(data)
   if x then
      x,err = load(x,name and name or "")
      if x then
         return x
      end
   end
   return nil,err
end

local function writeErr(response,type,msg)
   response:write("<pre style='color:red;font-size:120%'>")
   local from, line, err = string.match(msg,'%[string "([^"]+)"%]:(%d+):(.+)')
   if from and line and err then
      response:write(type,'.<br/>line: ',line,':<br/>',err)
   else
      response:write(type,".<br/>",msg)
   end
   response:write("</pre>")
end

local function lspPageFactory()
   local cachedPage
   local pageTab={}

   local function service(_ENV, data, lspType, compress, pageKey)
      local cachEnabled = lspType % 0x10
      local compEnabled = cachEnabled ~= lspType and true or false
      cachEnabled = cachEnabled == 2 and true or false
      if pageKey then
         response:setdefaultheaders()
         local function ep()
            response:write(fmt("<span page='%s' id='page%s'>",pageKey,pageKey))
            local func,err=parselsp(data, "Page "..pageKey)
            if func then
               local _,err = pcall(func,_ENV,relpath,app.io,pageTab,app)
               if err then
                  writeErr(response,"Executing Lua Code Failed",err)
               end
            else
               writeErr(response,"Compiling Lua Code Failed",err)
            end
            response:write("</span>")
            cachedPage=nil
         end
         cmsEmitPage = ep
         themeFunc(_ENV,relpath,app.io,themePage,app)
      else
         if cachEnabled then
            if not cachedPage then
               local function ep()
                  response:write("]===];",ba.parselsp(data,true),"_emit[===[")
               end
               local rspTab={"local _ENV,pathname,io,page,app=(function(a,b,c,d,e) return a,b,c,d,e end)(...);"}
               local function onResp(data)
                  table.insert(rspTab, data)
               end
               local xresp = response:setresponse(onResp)
               response:write("_emit[===[")
               cmsEmitPage = ep
               themeFunc(_ENV,relpath,app.io,themePage,app)
               response:write("]===];")
               xresp:finalize()
               response:reset()
               response:setdefaultheaders()
               local x,err=load(table.concat(rspTab))
               if err then
                  print(err)
                  return
               end
               cachedPage=x
            end
            if compEnabled then
               local xresp=response:setresponse()
               response:setdefaultheaders()
               cachedPage(_ENV,relpath,app.io,pageTab,app)
               xresp:finalize(true)
            else
               response:setdefaultheaders()
               cachedPage(_ENV,relpath,app.io,pageTab,app)
            end
         else
            local function ep()
               parselsp(data)(_ENV, relpath,app.io,pageTab,app)
            end
            cmsEmitPage = ep
            if compEnabled then
               local xresp=response:setresponse()
               response:setdefaultheaders()
               themeFunc(_ENV,relpath,app.io,themePage,app)
               xresp:finalize(true)
            else
               response:setdefaultheaders()
               themeFunc(_ENV,relpath,app.io,themePage,app)
            end
         end
      end
   end

   return service
end
local pageTab

local function mkUrl(domain, uri)
   return fmt("%s/%s",domain,uri):gsub("//","/")
end

local months = {
   "January",
   "February",
   "March",
   "April",
   "May",
   "June",
   "July",
   "August",
   "September",
   "October",
   "November",
   "December"}








------------------ RSS ---------------------------------


local function toRssDate(time)
   return os.date("%a, %d %b %Y %H:%M:%S GMT",time)
end

local function emitRssItem(title,url,description,category,date,maxlen)
   --Strip HTML tags
   title = string.gsub(title, "(]].->)", "]]")
   description = string.gsub(description, "(]].->)", "]]")
   if #description > maxlen then
      description=string.sub(description, 1, maxlen)
   end
   return fmt('<item><title><![CDATA[%s]]></title><link>http://%s</link><description><![CDATA[%s]]></description><category>%s</category><pubDate>%s</pubDate></item>',
     title,
     url,
     description,
     category,
     toRssDate(date))
end


-- Based on http://www.rssboard.org
local function emitRssFeed(_ENV, category)
   local title
   local description
   local ttl
   local maxlen
   local image
   local where
   if app.categories[category] then
      title = category
      local cur = conn:execute(fmt("SELECT DISTINCT category, description, ttl, maxlen, image FROM blogcategories WHERE category='%s'",category))
      if not cur then
         return
      end
      category, description, ttl, maxlen, image = cur:fetch()
      cur:close()
      where = fmt("WHERE category='%s'", category)
   elseif category == "blog" then -- blog = The 'all category' RSS feed
      where=""
   else
      return
   end
   local rss=app.rss
   local cur = conn:execute(fmt('SELECT date,category,subject,url,data FROM blog %s ORDER BY date DESC limit %s', where, rss.entries))
   if not cur then
      return
   end
   local pubDate
   local date,cat,subj,url,data = cur:fetch()
   if date then
      pubDate=date
   else
      if not app.rssDate then
         app.rssDate = os.time()
      end
      pubDate=app.rssDate
   end
   lastModified=tonumber(pubDate)
   pageFound=true
   if not xresp and not request:checktime(lastModified) then
      cur:close()
      return
   end
   response:setdateheader("Last-Modified", lastModified)
   contentType="text/xml;charset=utf-8"
   response:setcontenttype(contentType)
   if not description or #description == 0 then description=rss.description end
   if not ttl or #ttl == 0 then ttl=rss.ttl end
   if not maxlen or #maxlen == 0 then maxlen=rss.maxlen end
   if not image or #image == 0 then image=rss.image end
   if not title or #title == 0 then title=rss.title end
   pubDate=toRssDate(pubDate)
   pfmt('<?xml version="1.0" encoding="utf-8"?><rss version="2.0"><channel><title>%s</title><link>http://%s</link><description>%s</description><pubDate>%s</pubDate><lastBuildDate>%s</lastBuildDate><category>%s</category><generator>FuguHub</generator><ttl>%s</ttl>',
     title,
     domain,
     description,
     pubDate,
     pubDate,
     category,
     ttl)
   if #rss.copyright > 0 then
      pfmt('<copyright>%s</copyright>', rss.copyright)
   end
   if #image > 0 then
      pfmt('<image><title>%s</title><url>http://%s</url><link>http://%s</link></image>',
        category, -- title
        mkUrl(domain,image), -- url
        domain) -- link
   end
   local maxlen = tonumber(maxlen)
   if date then
      while date do
         write(emitRssItem(subj,mkUrl(domain,cmsDate2BlogURI(date,url)),data,category,date,maxlen))
         date,cat,subj,url,data = cur:fetch()
      end
   else
      write(emitRssItem(
        "The FuguHub Web Server",
        "FuguHub.com/",
        "<p>The category you have selected is empty so there is not much you can do here except check out the cool server powering this site.</p><img src='http://FuguHub.com/images/lsp_trans.gif'/>",
        "FuguHub",
        app.rssDate,
        maxlen))
   end
   cur:close()
   write("</channel></rss>")
end


------------------ end RSS engine ---------------------------------


local function doBlog(_ENV, uri)
   local year,month,url
   local q -- SQL query string
   cmsIsBlog=true

   -- A URI (minus /blog/) can be one of the following:
   -- (1) YYYY/MM/url   *Full listing.
   -- (2) YYYY/MM      *List all entries for the month in full
   -- (3) YYYY         *List only the subjects for the year
   -- (4) category     *List only the subjects for all entries in the category
   -- (5) Empty        *List the 10 last entries in full
   -- (6) xxxx.rss     *RSS feed request

   local x = string.gmatch(uri, "[^/]+") -- Convert URI into directory names
   x()-- skip blog/.
   year=x()   -- Extract /year/month/url from URI
   if year then
      month=x()
      if month then
         url=x()
         if url then
            cmsTitle=url
         end
      end
      if not tonumber(year) then
         if sfind(year, "%.rss$") then -- (6)
            pageFound=false -- Set later, if found
            if app.rss then
               emitRssFeed(_ENV, string.gsub(string.sub(year, 1, -5), "-", " "))
            end
            return -- Done
         end
         if month then -- Invalid request
            response:sendredirect("/blog/")
            return
         end
         -- (4)
         category = string.gsub(year, "-", " ")
         cmsTitle = category
         year=nil
      end
   end
   local briefQ="SELECT ROWID,date,subject,url FROM blog WHERE"
   local fullQ="SELECT ROWID,date,category,subject,url,name,data FROM blog"
   local isBrief
   local fullList
   if year then
      if month then -- (1) or (2)
         local q1 = fmt("%s WHERE strftime('%%Y%%m',date, 'unixepoch') = '%s%s'", fullQ, year, month)
         local q2 = " ORDER BY date DESC"
         if url then -- (1)
            fullList=true
            setShowAdmLinks()
            q=fmt("%s AND url = '%s' %s", q1, url, q2)
         else -- (2)
            q=q1..q2
         end
      else -- (3)
         q=fmt("%s strftime('%%Y',date, 'unixepoch') = '%s' ORDER BY date DESC",briefQ, year)
         isBrief=true
      end
   elseif category then -- (4)
      q=fmt("%s category='%s' ORDER BY date DESC",briefQ, category)
      if app.rss then
         cmsRssUri=fmt("/blog/%s.rss",category:gsub(' ','-'))
      end
      isBrief=true
   else --(5)
      q=fmt("%s ORDER BY date DESC limit 10", fullQ) 
      if app.rss then
         cmsRssUri="/blog/blog.rss"
      end
   end
   if app.rss then
      cmsHeader=cmsHeader..app.rssHtmlList
   end
   local cur = conn:execute(q)
   if cur then
      if isBrief then
         local rowid
         rowid,lastModified,subj,url = cur:fetch()
         if not lastModified then
            pageFound=false
         elseif xresp or not useCache or request:checktime(lastModified) then
            if not cmsTitle then
               cmsTitle= subj
            end
            cmsEmitPage = function()
               print("<table id='blogtab'>")
               local time = lastModified
               while time do
                  pageFound=true
                  pfmt("<tr><td>%s</td><td><a href='%s'>%s</a></td><td bid='%s'></td></tr>\n",
                       os.date("%A, %B %dth, %Y", time),
                       cmsDate2BlogURI(time,url),
                       subj,
                       rowid)
                  rowid,time,subj,url = cur:fetch()
               end
               print("</table>")
            end
            response:setdateheader("Last-Modified", lastModified)
            cmsHeader=cmsHeader.."<script src='/zzPreview.js' type='text/javascript'></script>"
            themeFunc(_ENV,relpath,app.io,themePage,app)
         end
      else -- Full listing
         local rowid,cat,name,data
          -- If admin: Wrap content into <span> elements so admin can edit.
         if cmsIsAdmin then
            -- The JavaScript using the span elements
            cmsTitle="Blog Administrator Mode"
            if not year then
               -- If URL is /blog/: Add <span> for making new blog.
               setShowAdmLinks()
               rowid="new"
               lastModified=os.time()
               cat=""
               subj=""
               name=""
               url="../.."
               data="<span id='hnew'></span><span blg='new' id='bnew'></span>"
            else
               rowid,lastModified,cat,subj,url,name,data = cur:fetch()
               if rowid then -- Edit.
                  subj = fmt('<span id="h%s">%s</span>',rowid,subj)
                  data = fmt('<span blg="%s" id="b%s">%s</span>',rowid,rowid,data)
               end
            end
         else -- not admin
            rowid,lastModified,cat,subj,url,name,data = cur:fetch()
            if not cmsTitle then
               cmsTitle= subj and subj or "blog"
            end
         end
         if xresp or not useCache or not lastModified or request:checktime(lastModified) then
            local time=lastModified
            local fbkey=#config.fbkey > 0 and config.fbkey or nil
            local fblink=#config.fblink > 0 and config.fblink or "Comments"
            cmsFbComments = function() -- Facebook comments
               if fbkey then
                  if fullList then
                     return fblink,fbkey
                  end
                  return fblink,nil
               end
               return nil
            end

            cmsBlog = function() -- Create function used by the theme
               if rowid then
                  local u
                  local r,d,c,s,n,dt = rowid,time,cat,subj,name,data
                  u=cmsDate2BlogURI(d,url)
                  rowid,time,cat,subj,url,name,data = cur:fetch()
                  pageFound=true
                  return u,r,d,c,s,n,dt 
               end
               return nil
            end
            if lastModified then
               response:setdateheader("Last-Modified", lastModified)
            end
            themeFunc(_ENV,relpath,app.io,themePage,app)
         end
      end -- End full listing
      cur:close()
   end
end


-- Returns true if browser accepts the deflate compression format
local function acceptsDeflate(_ENV)
   local ae = request:header("Accept-Encoding")
   return ae and ae:find("deflate") and true or false
end

local function sendDeflate(_ENV, data, lastModified,contentType)
   -- Remove default LSP headers such as Cache-Control, etc.
   response:reset()
   if lastModified then
      if not request:checktime(lastModified) then
         return
      end
      response:setdateheader("Last-Modified", lastModified)
   end
   if not contentType then
      contentType="text/html"
   end
   response:setcontenttype(contentType)
   response:setheader("Content-Encoding", "deflate")
   response:setcontentlength(#data)
   response:send(data)
end




------------------------- Start directory function ----------------------------
function cmsService(_ENV,relpath)

if response:committed() then return end
local method=request:method()
if method=="POST" then
   local ct=request:header"Content-Type"
   local uri=request:uri()
   if ct and ct:find"text/xml" and not request:uri():find"rtl/" then
      response:forward"/metaweblog/"
   end
end

local lastModified -- The last time the current page was modified.

-- Used when creating a cached page. Undefined if html i.e. the default.
local contentType

-- Set to true if we found the page
pageFound=false

cfg = app.cfg
config = app.config
cmsShowLogin = cfg.ShowLogin

if cfg.ssl and not request:issecure() then
   response:sendredirect(bd.getSecureUrl(request))
   return
end

-- Fetch data used by this application.

-- Fetch the session object, if any. We have a session object if user
-- is authenticated.
session = request:session(false) -- Do not create if we do not have a session

-- The username if user is authenticated
if session then
   cmsUname = request:user()
   if cmsUname then
      cmsIsAdmin = session.cmsIsAdmin
      if session.cmsIsAdmin == nil then
         session.cmsIsAdmin=false
      end
   end
end


fmt = string.format
function pfmt(...)
   write(fmt(...))
end

-- If we should use cache
local useCache=true
local xresp

domain=request:domain()
if not domain then
   domain = request:peername()
end
cmsHeader=fmt('<link rel="EditURI" type="application/rsd+xml" title="RSD" href="http://%s/metaweblog/rsd.lsp"/>%s',
              domain,
              '<script src="/rtl/jquery.js" type="text/javascript"></script><script src="/zzCMS.js" type="text/javascript"></script><script type="text/javascript">$(function(){if($(".PhotoAlbum ul li").length>0)$.getScript("/album/lightbox.js");});</script>')
if cmsUname and cmsIsAdmin == nil then
   useCache=false
   cmsHeader=cmsHeader.."<script src='/zzIsManager.js' type='text/javascript'></script>"

elseif cmsIsAdmin==true then
   useCache=false
   cmsHeader=cmsHeader.."<script src='/zzEdit.js' type='text/javascript'></script><script type='text/javascript'>var isManager=true;</script>"
end
cmsShowAdmLinks=false
function setShowAdmLinks()
   if cmsIsAdmin then
      cmsShowAdmLinks=true
   end
end

function cmsDate2BlogURI(date, url)
   return fmt("/blog/%s/%s",os.date("%Y/%m", date),url)
end

function cmsPostedIn(category)
   if category and #category ~= 0 then
      return fmt("Posted in <a href='/blog/%s'>%s</a>", category:gsub(' ','-'),category)
   end
   return nil
end

function cmsEmitGA()
   if cfg.GoogleAnalytics then
      write[[
<script>
  (function(i,s,o,g,r,a,m){i['GoogleAnalyticsObject']=r;i[r]=i[r]||function(){
  (i[r].q=i[r].q||[]).push(arguments)},i[r].l=1*new Date();a=s.createElement(o),
  m=s.getElementsByTagName(o)[0];a.async=1;a.src=g;m.parentNode.insertBefore(a,m)
  })(window,document,'script','//www.google-analytics.com/analytics.js','ga');
         ga('create', ']]
        write(config.GoogleAnalytics)
        write[[', 'auto');
  ga('send', 'pageview');
  </script>]]
   end
end 

function cmsMonth(m)
   return months[tonumber(m)]
end



-- Load new theme if changed or if first time called
if themeName ~= config.theme then
   local newTheme=config.theme
   local emsg="Cannot open theme "
   local bio = bd.baseio
   local name = "themes/"..newTheme
   themeFunc=nil
   themeName=nil
   themePage=nil
   if not bio:stat(name) then
      name=name..".zip"
      if not bio:stat(name) then
         print(emsg..newTheme)
         return
      end
   end
   local io,err = ba.mkio(bio, name)
   if io then
      local fp,err = io:open".layout.lsp"
      if fp then
         themeFunc,err = parselsp(fp:read"*a", ".layout.lsp")
         fp:close()
      end
   end
   if not themeFunc then
      print(fmt("%s%s: %s",emsg,newTheme,err and err or ""))
      return
   end
   if themeDir then
      themeDir:unlink()
   end
   themeDir = ba.create.resrdr("theme",io)
   themeDir:lspfilter(app)
   app.dir:insert(themeDir)
   themePage={}
   pageTab={}
   themeName = newTheme
end

if useCache then
   response:setheader"Cache-Control" -- Remove
end

uri = request:uri()
local doCompress -- Compress response: true/false

-- Logic for handling cached pages, if browser accepts compressed data.
if useCache and acceptsDeflate(_ENV) then
   local data
   data,lastModified,contentType = app.getCachedPage(uri)
   if data then
      sendDeflate(_ENV,data,lastModified,contentType)
      return
   end
   doCompress=true
end
function write(...)
   response:write(...)
end

-- Fix for Microsoft Word
if request:method() == "PROPFIND" and uri =="/" then
   response:write('<?xml version="1.0" encoding="utf-8"?><D:multistatus xmlns:D="DAV:"><D:response><D:href>',request:url(),'</D:href><D:propstat><D:status>HTTP/1.1 200 OK</D:status></D:propstat></D:response></D:multistatus>')
   return true
end

env,conn = app.openDb()
if env then
   if sfind(uri, "/blog/",1,true) == 1  then -- If URL starts with /blog/
      if doCompress then
         xresp=response:setresponse()
      end
      doBlog(_ENV, uri)
   elseif sfind(uri, "/MB/",1,true) == 1  then
      pageFound=app.mbDir(request,response,uri:sub(5))
   else -- end if blog
      local cur = conn:execute(string.format(
        "SELECT DISTINCT k,date,lname,data,lsp FROM links WHERE url='%s'",
        uri:gsub("'","''")))
      if cur then
         local pageKey,pageData,lspEnabled
         pageKey,lastModified,cmsTitle,pageData,lspAction = cur:fetch()
         lspAction=tonumber(lspAction)
         lastModified=tonumber(lastModified)
         cur:close()
         if pageKey then
            pageAction=pageEngine
            pageFound=true
            if app.rss then
               cmsHeader=cmsHeader..app.rssHtmlList
            end
            if cmsIsAdmin then
               setShowAdmLinks()
               if lspAction == 0 then
                  cmsEmitPage = function()
                     write(fmt("<span page='%s' id='page%s'>",
                                        pageKey,pageKey),pageData,"</span>")
                  end
                  themeFunc(_ENV,relpath,app.io,themePage,app)
               else
                  local service = pageTab[uri]
                  if not service then
                     service = lspPageFactory()
                     pageTab[uri]=service
                  end
                  service(_ENV,pageData,1,doCompress,pageKey)
               end
            elseif lspAction ~= 0 then
               local service = pageTab[uri]
               if not service then
                  service = lspPageFactory()
                  pageTab[uri]=service
               end
               service(_ENV,pageData,lspAction,doCompress)
            elseif doCompress or not useCache or request:checktime(lastModified) then
               if doCompress then
                  xresp=response:setresponse()
               end
               cmsEmitPage = function() write(pageData) end
               response:setdateheader("Last-Modified", lastModified)
               themeFunc(_ENV,relpath,app.io,themePage,app)
            end
         elseif uri == "/" then
            if cmsIsAdmin then
               response:sendredirect("/private/manage/PageManager.lsp?nomain=")
               pageFound=true
            end
         end
      end
   end
   app.closeDb(env,conn)
   if xresp then
      local data = xresp:finalize()
      if pageFound then
         if #data == 0 then
            return false -- 404 not found.
         end
         app.setCachedPage(data, uri, lastModified, contentType)
         sendDeflate(_ENV,data,lastModified,contentType)
         return true
      end
      response:reset()
   end
   if not pageFound then
      if xresp then xresp:abort() end
      return false -- 404 not found.
   end
else
   response:senderror(503, "DB connect"..(con or "?"))
end

return true
end -- End directory function

return cmsService
